﻿Imports System.Threading.Tasks
Imports System.Threading


Namespace Core

	''' <summary>
	''' A class that either starts new threads directly or 
	''' uses timers to to start threads indirectly.
	''' This version use the async and await language feature. 
	''' </summary>
	Public Class SystemUnderTestAsyncAwait
		Private timer As Timer
        Private ReadOnly lockObject As New Object
		Private throwExceptionFlag As Boolean
		Private nextTimerNotification As TimerNotification

		''' <summary>
		''' Start asynchronous operation with usage of async keyword
		''' </summary>
		''' <returns></returns>
		Public async Function DoWorkAsync() As Task
			Me.Message = "Init"
            Await Task.Run(
                Async Function()
                    If throwExceptionFlag Then
                        Throw New Exception
                    End If
                    Await Task.Delay(1500)
                    Me.Message &= " Work"
                End Function)
		End Function

		Public Function StartRecurring() As TimerNotification
			Me.Message = "Init"

            nextTimerNotification = New TimerNotification

			' Set up timer with 1 sec delay and 1 sec interval.
            Me.timer = New Timer(Sub(o)
                                     SyncLock lockObject
                                         Dim currentTimerNotification = nextTimerNotification
                                         nextTimerNotification = New TimerNotification
                                         Message &= " Poll"
                                         currentTimerNotification.SetNext(nextTimerNotification)
                                     End SyncLock
                                 End Sub, Nothing, New TimeSpan(0, 0, 0, 1), New TimeSpan(0, 0, 0, 1))

			Return nextTimerNotification
		End Function

		Public Sub StopRecurring()
			Me.timer.Change(Timeout.Infinite, Timeout.Infinite)
			Me.timer = Nothing
		End Sub

        Private PrivateMessage As String
        Public Property Message As String
            Get
                Return PrivateMessage
            End Get
            Private Set(ByVal value As String)
                PrivateMessage = value
            End Set
        End Property

		Public Sub PrepareToThrowException()
			throwExceptionFlag = True
		End Sub
	End Class
End Namespace
