﻿Imports System.Threading
Imports System.Threading.Tasks


Namespace Core

	''' <summary>
	''' A class that either starts new threads directly or 
	''' uses timers to to start threads indirectly. 
	''' This version uses events to notify clients
	''' about completion of asynchronous operations.  
	''' </summary>
	Public Class SystemUnderTestEvent
		Private rawTimer As Timer
		Private throwExceptionFlag As Boolean

		Public Event AsynchronousOperationDone As Action
		Public Event ReccuringOperationDone As Action

		''' <summary>
		''' Start asynchronous operation with firing event
		''' </summary>
		Public Sub StartAsynchronousOperation()
			Me.Message = "Init"

				' sometimes the asynchronous operation throws an exceptions 
            Task.Run(Sub()
                         If throwExceptionFlag Then
                             Throw New Exception
                         End If
                         Thread.Sleep(1500)
                         Message &= " Work"
                         OnAsynchronousOperationDone()
                     End Sub)
		End Sub

		Public Sub StartRecurring()
			' here goes any initialization
			Me.Message = "Init"

					' sometimes the asynchronous operation throws an exceptions 
            rawTimer = New Timer(Sub(o)
                                     If throwExceptionFlag Then
                                         Throw New Exception
                                     End If
                                     Message &= " Poll"
                                     OnReccuringOperationDone()
                                 End Sub, Nothing, New TimeSpan(0, 0, 0, 1), New TimeSpan(0, 0, 0, 1))
		End Sub

		Public Sub StopRecurring()
			rawTimer.Change(Timeout.Infinite, Timeout.Infinite)
			rawTimer = Nothing
		End Sub

        Private PrivateMessage As String
        Public Property Message As String
            Get
                Return PrivateMessage
            End Get
            Private Set(ByVal value As String)
                PrivateMessage = value
            End Set
        End Property

		Public Sub PrepareToThrowException()
			throwExceptionFlag = True
		End Sub


		Protected Overridable Sub OnAsynchronousOperationDone()
            Dim handler As Action = Sub()
                                        RaiseEvent AsynchronousOperationDone()
                                    End Sub
			If handler IsNot Nothing Then
				handler()
			End If
		End Sub

		Protected Overridable Sub OnReccuringOperationDone()
            Dim handler As Action = Sub()
                                        RaiseEvent ReccuringOperationDone()
                                    End Sub
			If handler IsNot Nothing Then
				handler()
			End If
		End Sub

	End Class
End Namespace
