﻿Imports System.Timers
Imports System.Threading.Tasks

Namespace Core

	''' <summary>
	''' A class that either starts new threads directly or 
	''' uses timers to to start threads indirectly.
	''' Some formerly private methods are now public to allow
	''' testing without multithreading. 
	''' </summary>
	Public Class SystemUnderTestHumbleLight
		Private timer As Timer
		Private throwExceptionFlag As Boolean


		''' <summary>
		''' Starts asynchronous operation which is implemented in Do1() and Do2().
		''' </summary>
		Public Sub StartAsynchronousOperation()
			Init()

            Dim task1 As Task = Task.Run(Sub()
                                             Me.Do1()
                                         End Sub)
            task1.ContinueWith(AddressOf Do2Internal, TaskScheduler.FromCurrentSynchronizationContext)
		End Sub

		Public Sub StartRecurring()
			Init()

			timer = New Timer(1000)
			AddHandler timer.Elapsed, AddressOf timer_Elapsed
			timer.Start()
		End Sub

		Public Sub StopRecurring()
			If timer IsNot Nothing Then
				timer.Stop()
				timer = Nothing
			End If
		End Sub

        Private PrivateMessage As String
        Public Property Message As String
            Get
                Return PrivateMessage
            End Get
            Private Set(ByVal value As String)
                PrivateMessage = value
            End Set
        End Property

		Public Sub PrepareToThrowException()
			throwExceptionFlag = True
		End Sub

		''' <summary>
		''' Changed to public to allow access for tests
		''' </summary>
		Public Sub Init()
			' here goes any initialization
			Me.Message = "Init"
		End Sub

		''' <summary>
		''' Changed to public to allow access for tests
		''' </summary>
		''' <exception cref="System.Exception"></exception>
		Public Sub Do1()
			' Sometimes the asynchronous operation throws an exceptions 
			If throwExceptionFlag Then
                Throw New Exception
			End If

			' here goes the first asynchronous operation
			Me.Message &= " Work"
		End Sub

		''' <summary>
		''' Changed to public to allow access for tests
		''' </summary>
		''' <param name="task"></param>
		Public Sub Do2(ByVal taskStatus As TaskStatus)
			' here goes the second asynchronous operation
			If taskStatus = TaskStatus.RanToCompletion Then
				Me.Message &= " OK"
			End If
			If taskStatus = TaskStatus.Faulted Then
				Me.Message &= " NOK"
			End If
		End Sub

		''' <summary>
		''' Changed to public to allow access for tests
		''' </summary>
		Public Sub DoRecurring()
			If throwExceptionFlag Then
                Throw New Exception
			End If

			Me.Message &= " Poll"
		End Sub


		Private Sub Do2Internal(ByVal task As Task)
			Do2(task.Status)
		End Sub

		Private Sub timer_Elapsed(ByVal sender As Object, ByVal e As ElapsedEventArgs)
			Me.DoRecurring()
		End Sub

	End Class
End Namespace
