﻿Imports System.Timers

Imports System.Threading.Tasks

Namespace Core

	''' <summary>
	''' A class that either starts new threads directly or 
	''' uses timers to to start threads indirectly.
	''' The "HubleObject" refactoring is applied and the functionality
	''' is extracted.
	''' </summary>
	Public Class SystemUnderTestHumbleObject
		Private ReadOnly functionality As Functionality
		Private timer As Timer

		Public Sub New()
            Me.functionality = New Functionality
		End Sub

		''' <summary>
		''' Starts asynchronous operation which is implemented in the Functionality class.
		''' </summary>
		Public Sub StartAsynchronousOperation()
			functionality.Init()

            Dim task1 As Task = Task.Run(Sub()
                                             Me.Do1()
                                         End Sub)
            task1.ContinueWith(AddressOf Do2, TaskScheduler.FromCurrentSynchronizationContext)
		End Sub

		Public Sub StartRecurring()
			' here goes any initialization
			functionality.Init()

			timer = New Timer(New TimeSpan(0,0,0,1).Milliseconds)
			AddHandler timer.Elapsed, AddressOf timer_Elapsed
			timer.Start()
		End Sub

		Public Sub StopRecurring()
			If timer IsNot Nothing Then
				timer.Stop()
				timer = Nothing
			End If
		End Sub

        Public ReadOnly Property Message As String
            Get
                Return functionality.Message
            End Get
        End Property

		Private Sub Do1()
			' Call the first asynchronous operation
			functionality.Do1()
		End Sub

		Private Sub Do2(ByVal task As Task)
			' Call the second asynchronous operation
			functionality.Do2(task.Status)
		End Sub


		Private Sub timer_Elapsed(ByVal sender As Object, ByVal e As ElapsedEventArgs)
			functionality.DoRecurring()
		End Sub

	End Class
End Namespace
