﻿Imports System.Threading

Imports System.Threading.Tasks

Namespace Core

	''' <summary>
	''' A class that either starts an asynchronous operation directly or 
	''' uses timers to start an asynchronous operation indirectly.
	''' The class uses a supplied TaskScheduler or a supplied ITimer.
	''' </summary>
	Public Class SystemUnderTestTaskSchedulerTimer
		Private ReadOnly taskScheduler As TaskScheduler
		Private ReadOnly timer As ITimer
		Private throwExceptionFlag As Boolean
        Private ReadOnly lockObject As New Object

		Public Sub New(ByVal taskScheduler As TaskScheduler)
			Me.taskScheduler = taskScheduler
		End Sub

		Public Sub New(ByVal timer As ITimer)
			Me.timer = timer
		End Sub

		''' <summary>
		''' Start asynchronus operation with supplied TaskScheduler
		''' </summary>
		Public Sub StartAsynchronousOperation()
			' here goes any initialization
			Me.Message = "Init"

            Dim task1 As Task = Task.Factory.StartNew(Sub()
                                                          If throwExceptionFlag Then
                                                              Throw New Exception
                                                          End If
                                                          Message &= " Work1"
                                                      End Sub, CancellationToken.None, TaskCreationOptions.None, taskScheduler)
            task1.ContinueWith(Sub(t) Message &= " Work2", taskScheduler)
		End Sub

		Public Sub StartRecurring()
			Me.Message = "Init"

            timer.StartTimer(Sub()
                                 SyncLock lockObject
                                     If throwExceptionFlag Then
                                         Throw New Exception
                                     End If
                                     Me.Message &= " Poll"
                                 End SyncLock
                             End Sub, New TimeSpan(0, 0, 0, 1))
		End Sub

		Public Sub StopRecurring()
			timer.StopTimer()
		End Sub

        Private PrivateMessage As String
        Public Property Message As String
            Get
                Return PrivateMessage
            End Get
            Private Set(ByVal value As String)
                PrivateMessage = value
            End Set
        End Property

		Public Sub PrepareToThrowException()
			throwExceptionFlag = True
		End Sub
	End Class
End Namespace
