﻿Imports Core


Namespace Tests

	''' <summary>
	''' A timer to be used during unit testing. It allows to start timer operations
	''' on the same thread as the unit test.
	''' </summary>
	Public Class DeterministicTimer
		Implements ITimer
		Private timerAction As Action
		Private timerInterval As TimeSpan

		Private isStarted_Renamed As Boolean
		Private elapsedTime As TimeSpan

		#Region "ITimer methods"
		Public Sub StartTimer(ByVal action As Action, ByVal interval As TimeSpan) Implements ITimer.StartTimer
			isStarted_Renamed = True
			timerAction = action
			timerInterval = interval
            elapsedTime = New TimeSpan
		End Sub

		Public Function IsStarted() As Boolean Implements ITimer.IsStarted
			Return isStarted_Renamed
		End Function

		Public Sub StopTimer() Implements ITimer.StopTimer
			isStarted_Renamed = False
		End Sub
		#End Region

		''' <summary>
		''' Tell the timer that some seconds have elapsed and let the timer execute the timer action. 
		''' </summary>
		''' <param name="seconds"></param>
		Public Sub ElapseSeconds(ByVal seconds As Integer)
			Dim newElapsedTime As TimeSpan = elapsedTime.Add(New TimeSpan(0, 0, 0, seconds))
			If isStarted_Renamed Then
				Dim executionCountBefore As Long = elapsedTime.Ticks \ timerInterval.Ticks
				Dim executionCountAfter As Long = newElapsedTime.Ticks \ timerInterval.Ticks

                For i As Integer = 0 To CInt(executionCountAfter - executionCountBefore - 1)
                    timerAction()
                Next i
			End If
			elapsedTime = newElapsedTime
		End Sub
	End Class
End Namespace
