﻿Imports NUnit.Framework
Imports Core
Imports System.Windows.Threading
Imports System.Threading
Imports System.Security.Permissions

Namespace Tests



	<TestFixture>
	Public Class TestsInitial
		<Test>
		Public Sub FragileAndSlowTest()
			' Setup
            Dim sut = New SystemUnderTestInitial

			' Operation to be tested will run in another thread.
			sut.StartAsynchronousOperation()

			' Bad Idea: Hoping that the other thread finishes execution after 2 seconds.
			Thread.Sleep(2000)

			' Assert outcome of the thread.
			Assert.AreEqual("Init Work", sut.Message)
		End Sub

		<Test>
		Public Sub FragileAndSlowTestWithSystemThreadingTimer()
			' Setup
            Dim sut = New SystemUnderTestInitial

			' Execute code to set up timer with 1 sec delay and interval:
			sut.StartRecurring()
			' Bad idea: Wait for timer to trigger three times.
			Thread.Sleep(3100)
			sut.StopRecurring()

			' Assert outcome
			Assert.AreEqual("Init Poll Poll Poll", sut.Message)
		End Sub

		<Test>
		Public Sub FragileAndSlowTestWithDispatcherTimer()
			' Setup
            Dim sut = New SystemUnderTestInitialWithDispatcherTimer

			' Execute code to set up timer with 1 sec delay and interval:
			sut.StartRecurring()

			' Bad idea: Wait for timer to trigger three times while 
			Dim waitMilliseconds As Integer = 0
			Do While waitMilliseconds < 3500
				DispatcherUtil.DoEvents()
				Thread.Sleep(200)
				waitMilliseconds += 200
			Loop
			sut.StopRecurring()

			' Assert outcome
			Assert.AreEqual("Init Poll Poll Poll", sut.Message)
		End Sub

	End Class

	Public NotInheritable Class DispatcherUtil

		<SecurityPermissionAttribute(SecurityAction.Demand, Flags := SecurityPermissionFlag.UnmanagedCode)>
		Public Shared Sub DoEvents()
            Dim frame As New DispatcherFrame
			Dispatcher.CurrentDispatcher.BeginInvoke(DispatcherPriority.Background, New DispatcherOperationCallback(AddressOf ExitFrame), frame)
			Dispatcher.PushFrame(frame)
		End Sub

		Private Shared Function ExitFrame(ByVal frame As Object) As Object
			CType(frame, DispatcherFrame).Continue = False
			Return Nothing
		End Function
	End Class
End Namespace
