﻿Imports Core
Imports NUnit.Framework
Imports System.Threading.Tasks


Namespace Tests

	<TestFixture>
	Public Class TestsSolution2AsyncAwait
		''' <summary>
		''' Synchronizes the test with code via asynchronous await.
		''' Test needs to be executed with NUnit Runner 2.6.2 or later.
		''' </summary>
		<Test>
		Public async Function SynchronizeTestWithCodeViaAwait() As Task
			' Setup
            Dim sut = New SystemUnderTestAsyncAwait

			' Schedule operation to run asynchronously and wait until it is finished.
            Await sut.DoWorkAsync

			' Assert
			Assert.AreEqual("Init Work", sut.Message)
		End Function


		''' <summary>
		''' Synchronizes the test with code via returned task. Use this in case
		''' the test runner cannot handle "public async Task" test method signatures.
		''' </summary>
		<Test>
		Public Sub SynchronizeTestWithCodeViaReturnedTask()
			' Setup
            Dim sut = New SystemUnderTestAsyncAwait

			' Schedule operation to run asynchronously and wait until it is finished.
            Dim task As Task = sut.DoWorkAsync
			task.Wait()

			' Assert
			Assert.AreEqual("Init Work", sut.Message)
		End Sub


		''' <summary>
		''' Synchronizes the test with code via asynchronous await.
		''' The test expects that the method throws an exception.
		''' Test needs to be executed with NUnit Runner 2.6.2 or later.
		''' </summary>
		<Test, ExpectedException(GetType(Exception))>
		Public async Function SynchronizeTestWithCodeViaAsyncAwaitAndExpectException() As Task
			' Setup
            Dim sut = New SystemUnderTestAsyncAwait
			sut.PrepareToThrowException()

			' Execute
            Await sut.DoWorkAsync
		End Function

		<Test>
		Public async Function SynchronizeTestWithRecurringOperationViaAwait() As Task
			' Setup
            Dim sut = New SystemUnderTestAsyncAwait

			' Execute code to set up timer with 1 sec delay and interval.
            Dim firstNotification = sut.StartRecurring

			' Wait that operation has finished two times
            Dim secondNotification = Await firstNotification.GetNext
            Await secondNotification.GetNext
			sut.StopRecurring()

			' Assert
			Assert.AreEqual("Init Poll Poll", sut.Message)
		End Function
	End Class
End Namespace
