﻿Imports Rhino.Mocks
Imports Core
Imports NUnit.Framework
Imports System.Threading.Tasks
Imports System.Threading

Namespace Tests


	<TestFixture>
	Public Class TestsSolution2Depentent
		<Test>
		Public Sub SynchronizeTestWithCodeViaDependentObjectAndThreadSynchonizationEvent()
			' Setup
			Dim dependentStub = MockRepository.GenerateStub(Of ISomeInterface)()
			Dim sut = New SystemUnderTestDependent(dependentStub)
            Dim finishedEvent = New ManualResetEventSlim
            dependentStub.Stub(Sub(x) x.DoMore()).WhenCalled(Sub(x) finishedEvent.Set())

			' Schedule operation to run asynchronously.
			sut.StartAsynchronousOperation()

			' Wait for operation to be finished for maximum 2 sec.
			finishedEvent.Wait(TimeSpan.FromSeconds(2))

			' Assert outcome of operation
			Assert.AreEqual("Init Work", sut.Message)
		End Sub

		<Test>
		Public async Sub SynchronizeTestWithCodeViaDependentObjectAndTaskCompletionSource()
			' Setup
			Dim dependentStub = MockRepository.GenerateStub(Of ISomeInterface)()
			Dim sut = New SystemUnderTestDependent(dependentStub)
            Dim completionSource = New TaskCompletionSource(Of Boolean)
            dependentStub.Stub(Sub(x) x.DoMore()).WhenCalled(Sub(x) completionSource.SetResult(True))

			' Schedule operation to run asynchronously.
			sut.StartAsynchronousOperation()

			' Wait for operation to be finished for maximum 2 sec.
			await completionSource.Task

			' Assert outcome of operation
			Assert.AreEqual("Init Work", sut.Message)
		End Sub


		<Test>
		Public Sub SynchronizeTestWithCodeViaDependentObjectAndExceptionIsThrown()
			' Setup
			Dim dependentStub = MockRepository.GenerateStub(Of ISomeInterface)()
			Dim sut = New SystemUnderTestDependent(dependentStub)
			sut.PrepareToThrowException()
            Dim finishedEvent = New ManualResetEventSlim
            dependentStub.Stub(Sub(x) x.DoMore()).WhenCalled(Sub(x) finishedEvent.Set())

			' Execute
			sut.StartAsynchronousOperation()
			' Wait to be finsihed for maximum 2 sec.
			finishedEvent.Wait(TimeSpan.FromSeconds(2))

			' Assert (we cannot detect exception here because it is thrown in another thread)
			' Although the execution is aborted immediately, the test always takes 10 seconds.
			Assert.AreEqual("Init", sut.Message,"Only Init is done, work is aborted due to exception")
		End Sub

		<Test>
		Public Sub SynchronizeTestWithTimerViaDependentObject()
			' Setup
			Dim dependentStub = MockRepository.GenerateStub(Of ISomeInterface)()
			Dim sut = New SystemUnderTestDependent(dependentStub)
            Dim finishedEvent = New ManualResetEventSlim
            dependentStub.Stub(Sub(x) x.DoMore()).WhenCalled(Sub(x) finishedEvent.Set())

			' Execute
			sut.StartRecurring()
			' Wait to be finsihed for maximum 2 sec.
			finishedEvent.Wait(TimeSpan.FromSeconds(2))
			sut.StopRecurring()

			' Assert
			Assert.AreEqual("Init Poll", sut.Message)
		End Sub

		<Test>
		Public Sub SynchronizeTestWithTimerViaDependentObjectAndExpectException()
			' Setup
			Dim dependentStub = MockRepository.GenerateStub(Of ISomeInterface)()
			Dim sut = New SystemUnderTestDependent(dependentStub)
			sut.PrepareToThrowException()
            Dim finishedEvent = New ManualResetEventSlim
            dependentStub.Stub(Sub(x) x.DoMore()).WhenCalled(Sub(x) finishedEvent.Set())

			' Execute
			sut.StartRecurring()
			' Wait to be finsihed for maximum 2 sec.
			finishedEvent.Wait(TimeSpan.FromSeconds(2))

			' Assert
			Assert.AreEqual("Init", sut.Message, "Only Init succeded, because of exception")
		End Sub

	End Class
End Namespace
