﻿Imports Core
Imports NUnit.Framework
Imports System.Threading

Namespace Tests

	<TestFixture>
	Public Class TestsSolution2Event
		<Test>
		Public Sub SynchronizeTestWithCodeViaEvent()
			' Setup
            Dim sut = New SystemUnderTestEvent
            Dim finishedEvent = New ManualResetEventSlim
            AddHandler sut.AsynchronousOperationDone, AddressOf finishedEvent.Set
			' Execute
			sut.StartAsynchronousOperation()
			' Wait to be finsihed for maximum 10 sec.
			finishedEvent.Wait(TimeSpan.FromSeconds(10))

			' Assert
			Assert.AreEqual("Init Work", sut.Message)
		End Sub

		<Test>
		Public Sub SynchronizeTestWithCodeViaEventAndExpectException()
			' Setup
            Dim sut = New SystemUnderTestEvent
            Dim finishedEvent = New ManualResetEventSlim
            AddHandler sut.AsynchronousOperationDone, AddressOf finishedEvent.Set
			sut.PrepareToThrowException()

			' Execute
			sut.StartAsynchronousOperation()
			' Wait to be finsihed for maximum 10 sec.
			finishedEvent.Wait(TimeSpan.FromSeconds(10))

			' Assert
			Assert.AreEqual("Init", sut.Message, "Only Init succeded, because of exception")
		End Sub

		<Test>
		Public Sub SynchronizeTestWithTimerViaEvent()
			' Setup
            Dim sut = New SystemUnderTestEvent
            Dim finishedEvent = New ManualResetEventSlim
            AddHandler sut.ReccuringOperationDone, AddressOf finishedEvent.Set

			' Execute
			sut.StartRecurring()
			' Wait to be finsihed for maximum 10 sec.
			finishedEvent.Wait(TimeSpan.FromSeconds(10))
			sut.StopRecurring()

			' Assert
			Assert.AreEqual("Init Poll", sut.Message)
		End Sub

		<Test>
		Public Sub SynchronizeTestWithTimerViaEventAndExpectException()
			' Setup
            Dim sut = New SystemUnderTestEvent
            Dim finishedEvent = New ManualResetEventSlim
            AddHandler sut.ReccuringOperationDone, AddressOf finishedEvent.Set
			sut.PrepareToThrowException()

			' Execute
			sut.StartRecurring()
			' Wait to be finsihed for maximum 10 sec.
			finishedEvent.Wait(TimeSpan.FromSeconds(10))

			' Assert
			Assert.AreEqual("Init", sut.Message, "Only Init succeded, because of exception")
		End Sub

	End Class
End Namespace
