﻿Imports Core
Imports NUnit.Framework

Namespace Tests

	Public Class TestsSolution3Deterministic
		<Test>
		Public Sub TestCodeSynchronously()
			' Setup
            Dim dts = New DeterministicTaskScheduler
			Dim sut = New SystemUnderTestTaskSchedulerTimer(dts)

			' Execute code to schedule first task and return immediately.
			sut.StartAsynchronousOperation()

			' Execute all tasks on the current thread
			dts.RunTasksUntilIdle()

			' Assert
			Assert.AreEqual("Init Work1 Work2", sut.Message)
		End Sub

		<Test>
		Public Sub TestCodeSynchronouslyAndExpectException()
			' Setup
            Dim dts = New DeterministicTaskScheduler
			Dim sut = New SystemUnderTestTaskSchedulerTimer(dts)
			sut.PrepareToThrowException()

			' Execute code to create task and return immediately 
			sut.StartAsynchronousOperation()

			' Execute all tasks on the current thread. We expect that an exception is thrown
			Assert.That(AddressOf dts.RunTasksUntilIdle, Throws.TypeOf(Of Exception)())
		End Sub

		''' <summary>
		''' VeryFastAndReliableTestWithTimer
		''' </summary>
		<Test>
		Public Sub TestCodeWithTimerSynchronously()
			' Setup
            Dim dt = New DeterministicTimer
			Dim sut = New SystemUnderTestTaskSchedulerTimer(dt)

			' Execute
			sut.StartRecurring()

			' Tell timer that some time has elapsed
			dt.ElapseSeconds(3)
			sut.StopRecurring()

			' Assert that outcome of three executions of the recurring operation is ok.
			Assert.AreEqual("Init Poll Poll Poll", sut.Message)
		End Sub

		<Test>
		Public Sub TestCodeWithTimerSynchronouslyAndExpectException()
			' Setup
            Dim dt = New DeterministicTimer
			Dim sut = New SystemUnderTestTaskSchedulerTimer(dt)
			sut.PrepareToThrowException()

			' Execute
			sut.StartRecurring()

			' Tell timer that some time has elapsed. We expect that an exception is thrown
			Assert.That(Sub() dt.ElapseSeconds(3), Throws.TypeOf(Of Exception)())
		End Sub


	End Class
End Namespace
